(module top (lib "eopl.ss" "eopl")
  
  ;; top level module.  Loads all required pieces.
  ;; Run the test suite with (run-all).

  (require "drscheme-init.scm")
  (require "data-structures.scm")  ; for expval constructors
  (require "lang.scm")             ; for scan&parse
  (require "interp.scm")           ; for value-of-program
  (require "tests.scm")            ; for test-list
  
  (provide run run-all)
  
  
  ;;;;;;;;;;;;;;;; interface to test harness ;;;;;;;;;;;;;;;;
  
  ;; run : String -> ExpVal

  (define run
    (lambda (string)
      (value-of-program (scan&parse string))))
  
  ;; run-all : () -> Unspecified

  ;; runs all the tests in test-list, comparing the results with
  ;; equal-answer?  

  (define run-all
    (lambda ()
      (run-tests! run equal-answer? test-list)))
  
  (define equal-answer?
    (lambda (ans correct-ans)
      (equal? ans (sloppy->expval correct-ans))))
  
  (define sloppy->expval 
    (lambda (sloppy-val)
      (cond
        ((number? sloppy-val) (num-val sloppy-val))
        ((boolean? sloppy-val) (bool-val sloppy-val))
        ((list? sloppy-val) (list-val (map sloppy->expval sloppy-val)))
        (else
         (eopl:error 'sloppy->expval 
                     "Can't convert sloppy value to expval: ~s"
                     sloppy-val)))))
    
  ;; run-one : Sym -> ExpVal

  ;; (run-one sym) runs the test whose name is sym
  
  (define run-one
    (lambda (test-name)
      (let ((the-test (assoc test-name test-list)))
        (cond
          ((assoc test-name test-list)
           => (lambda (test)
                (run (cadr test))))
          (else (eopl:error 'run-one "no such test: ~s" test-name))))))

  ;;  (run-all)

  (display (run "
class node extends object
field next
field val
method initialize () 0
method getNext () next
method getVal () val
method setNext (newNext) set next = newNext
method setVal (newVal) set val = newVal
class queue extends object
field first
field last
field size
method initialize () set size = 0
method empty? () zero?(size)
method enqueue (val)
  let newnode = new node() in
    begin
      send newnode setVal(val);
      if send self empty?() 
      then
       begin
        set first = newnode;
        set last = first
       end
      else
        begin
          send last setNext(newnode);
          set last = newnode
        end;
      set size = +(size,1)
    end
method dequeue ()
  let val = send first getVal() in
    begin
      set first = send first getNext();
      set size = -(size,1);
      val
    end
let q = new queue() in begin send q enqueue (0);  send q enqueue (1); send q enqueue (2); send q dequeue() end
"))
  
  )




